
/**
 * Feed
 * */
function Feed( url, items, params ) { 
	
	var params = params ? params : {};
	this.url = url;
	this.items = [];
	this.lastUpdate = new Date(0);
	this.unreadNum = 0;
	this.title = params.title ? params.title : '';
	this.description = params.description ? params.description : '';
	this.icon = params.icon ? params.icon : '';
	this.color = params.color ? params.color : '';
	this.ITEMS_LIMIT = 10;
	
	if(items){
		this.setItems(items);
	}
	this.feedXHR = new XMLHttpRequest();
}
/**
 * 
 * */
Feed.prototype.setTimeLimit = function(ms){
	TIME_LIMIT = ms;
}

/**
 * fill items array with new ones
 * @param {Array} of items
 * */
Feed.prototype.setItems = function(items){
	
	items = this.cutExcess(items);
	this.items = [];
	this.unreadNum = 0;
	for( var i=0; i<items.length; i++ ) {
		items[i].setFeed( this );
		this.items.push( items[i] );
		if( !items[i].read ){
			this.unreadNum++;
		}
	}
	this.loaded = true;
};

/**
 * 
 * */
Feed.prototype.cutExcess = function(items){
	items.sort( function(a,b){if(a.pubDate > b.pubDate){return -1;}if(a.pubDate < b.pubDate){return 1;}return 0;} );
	return items.splice(0,this.ITEMS_LIMIT);
}

/**
 * set lastUpdate time
 * @param {Date} time
 * */
Feed.prototype.setLastUpdate = function(time){
	
	this.lastUpdate = time;
};

/**
 * called to get or refresh feed items and attributes
 * @method load
 * @param {Function} onSuccess
 * @param {Function} onFailure
 * @return {XMLHttpRequest} sent request
 */

Feed.prototype.load = function(onSuccess, onFailure){
	
	var feed = this;
	var t = setTimeout( function(){
		this.feedXHR.abort();
		onFailure.call( feed, "Unable to load feed. Time limit exceeded.");
	}, 30*60*100 );
	
	if(!this.url){
		throw new Error( "Unable to load feed. No url." );
	}
	
	this.feedXHR.onreadystatechange = function() {
        if( this.readyState == 4 ) {
            if( this.status == 200 || this.status == 304 ){
            	try {
                	var items = feed._parseResponse( this.responseXML );
            		feed.mergeItems(items); 
            		feed.lastUpdate = new Date(); 
                	feed.loaded = true;
    			}
    			catch(err){
    				if( t )clearTimeout(t);
    				onFailure.call(feed,err);
    			}
    			
    			if(onSuccess){
    				if( t )clearTimeout(t);
    				onSuccess.call(feed);
    			}

            }
            else {
    			if(onFailure){
    				if( t )clearTimeout(t);
    				onFailure.call(feed,"Unable to load feed.");
    			}
            }
        }
    };
    try{
    	
    	this.feedXHR.open( 'GET', this.url );
    	this.feedXHR.send(null);
    } catch(err){
    	if( t )clearTimeout(t);
    	onFailure.call(this, err );
    }
	return this.feedXHR;
};

/**
 * parses items from responseXML
 * @method _parseResponse
 * @private
 * @param {XMLObject} responseXML
 */
Feed.prototype._parseResponse = function( responseXML ){
    try{
    	var _items = Parser.getItems( responseXML );
    	
    	var _channel = Parser.getChannelData( responseXML );
    	if( _channel ){
    		if( _channel.icon ){
    			this.icon = _channel.icon;
    		}
    		if( _channel.title ){
    			this.title = _channel.title;
    		}
    		if( _channel.description ){
    			this.description = _channel.description;
    		}
    	}
    } catch(err){
    	throw err;
    }
	return _items;
};

/**
 * get feed and retrieves items if it's not loaded yet
 * @method get
 * @param {Function} onReady
 */
Feed.prototype.get = function(onReady){
	
	if(!this.loaded){
		this.load(onReady, onReady);
	}
	else {
		onReady.call(this);
	}
};

/**
 * merge new items with old ones, does not overwrite existing items and removes old items
 * @method mergeItems
 * @param {Array} items
 */
Feed.prototype.mergeItems = function(items){
	var feed = this,
		foundItem,
		item,
		_items = [],
		unreadNum = 0;
	
	for( var i=0; i<items.length; i++ ){
		item = items[i];
		foundItem = feed.find(item);
		if( foundItem ){
			_items.push(foundItem);
		} 
		else {
			item.setFeed(this);
			_items.push(item);
			if( !item.read ){
				unreadNum++;
			}
		}
	}
	this.setItems( _items );
	this.unreadNum = unreadNum;
};
/**
 * finds item with item provided
 * @method find
 * @param {Item} item
 */
Feed.prototype.find = function(item){
	
	for(var i = 0; i < this.items.length; i++){
		if(item.id == this.items[i].id){
			return this.items[i];
		}
	}
	return null;
}

/**
 * */
Feed.prototype.getUnreadNum = function(){
	var num = 0;
	
	for(var i = 0; i < this.items.length; i++){
		if(!this.items[i].read){
			num++;
		}
	}
	this.unreadNum = num;
	return num;
}

/**
 * returns items of feed
 * */
Feed.prototype.getItems = function(){
	return this.items;
};

/**
 * 
 * */
Feed.prototype.getProperties = function(){
	var temp = {
			url: this.url,
			name: this.name,
			lastUpdate: this.lastUpdate.getTime(),
			description: this.description,
			icon: this.icon,
			title: this.title,
			color: this.color,
			items: []
	};
	for(var i=0;i<this.items.length;i++){
		temp.items.push(this.items[i].getProperties());
	}
	
	return temp;
}

/**
 * 
 * */
Feed.create = function(properties){

	//opera.postError( JSON.stringify( properties ));
	var name = properties.name;
	var url  = properties.url;
	var items = [];
	
	for( var i=0; i<properties.items.length;i++ ){
		items.push(Item.prototype.create(properties.items[i]));
	}
	
	
	var d = new Date();
	d.setTime( properties.lastUpdate );

	var feed = new Feed(url,items, { 
		title: properties.title, 
		description: properties.description,
		icon: properties.icon,
		color: properties.color
	});
	feed.setLastUpdate(d);
	return feed;
}
