Application.Reader = (function(){
	
	/**
	 * stores list of active requests so they can be aborted easily
	 * @property activeRequests
	 * @type {Array}
	 */
	var activeRequests = [],
	
	/**
	 * stores all feeds objects
	 * @property feeds
	 * @type {Array}
	 */
		feeds = [];	
	
	/**
	 * @method 
	 * @private
	 */
	function getFeed(feed,onResult){
		var xhr = feed.load(function(){
			requestFinished(xhr);
			if(onResult)onResult(this);
	    },function(err){
			requestFinished(xhr);
			if(onResult)onResult(this,err);
		});
		activeRequests.push(xhr);
		return feed;
	}		
	
	/**
	 * 
	 * */
	function getFeeds(f){
        var fds = f || feeds,
        	feed,foundFeed;
        if(!fds.length){
        	Application.UI.overlay.showErrorMsg({
				title: 'No feeds', 
				message: "Your list of RSS feeds is empty. Add an RSS channel first."
			});
        	return;
        }
        Application.UI.overlay.showLoader( {title: 'Wait...', message: 'Reading RSS channels...'} );
        for(var i = 0; i < fds.length; i++){
        	feed = fds[i];
			foundFeed = findFeed(feed);
			if(!foundFeed){
				feeds.push(feed);
			}
            getFeed( feed, onFeedsResult );
        }
	}
	
	/**
	 * 
	 * */
	function findFeed( feed ){
		for(var i = 0; i < feeds.length; i++){
			if(feed.url == feeds[i].url){
				return feeds[i];
			}
		}
		
		return null;
	}
	
	/**
	 * callback for getFeeds, invoked after retriving data from one feeds
	 * @method onFeedsResult
	 * @private
	 * @param {Array} results
	 */
	function onFeedsResult(feed,err){
		
		var items = [];
		if( !activeRequests.length ) {
			var items = [];
			for( var i=0;i < feeds.length; i++){
				items = items.concat(feeds[i].getItems());
			}
			Application.UI.overlay.hideLoader();
			Application.UI.reader.showItems(items);		
		}
		save();
	}
	
	/**
	 * 
	 * */
	function requestFinished(request){
		
		var index = activeRequests.indexOf(request);
		if(index != -1){
			activeRequests.splice(index,1);
		}
	}
	
	/**
	 * save(): saving current feedlist to preferences
	 * */
	function save(){

		var saveFeeds = [];
		for(var i = 0; i < feeds.length; i++){
			saveFeeds.push(feeds[i].getProperties());
		}		
		if( window.localSorage != 'undefined' ){
			window.localStorage.setItem('feeds',JSON.stringify(saveFeeds));
		}
		else {
			widget.setPreferenceForKey( JSON.stringify(saveFeeds),'feeds')
		}
	}
	
	function htmlize( s ){
		return s.replace(/\x26/g,"&amp;");
	}
	
	/**
	 * exportFeeds(): 
	 * */
	function exportFeeds(){

		if( !feeds.length ) {
			Application.UI.overlay.showErrorMsg({
				title: 'Export failed',
				message: "There is no subscriptions to export.",
			});
			return;
		}
		try {
			opera.io.filesystem.browseForSave('', 'universal-rss-reader-subscriptions.xml',
				function(fo) {
					if(fo === null) { return; }
					try {
						var stream = fo.open(null, 'w');
	                  
						stream.writeLine( '<?xml version="1.0" encoding="UTF-16"?>' );
						stream.writeLine( '<opml version="1.0">' );
						stream.writeLine( '<head>' );
						stream.writeLine( '<title>RSS chanels from Opera Widget Uniwersal RSS READER</title>' );
						stream.writeLine( '</head>' );
						stream.writeLine( '<body>' );
	                      
						for(var i = 0; i < feeds.length; i++){
							stream.writeLine( '<outline text="' + 
											  (feeds[i].title ? htmlize(feeds[i].title) : 'Unknown') +
											  '" xmlUrl="' + htmlize(feeds[i].url) +
											  '" />'
											  );
						}
						stream.writeLine( '</body>' );
						stream.writeLine( '</opml>' );
						stream.close();
						
						Application.UI.overlay.showInfoDialog({
							title: 'Export successfull',
							message: "Your subscriptions has beed exported."
						});
					} catch( err ){
						Application.UI.overlay.showErrorMsg({
							title: 'Export failed', 
							message: err.message ? err.message : 'Some error occured when trying to import your subscriptions to file.'
						});
					}
          	});        
		} catch(err) {
			Application.UI.overlay.showErrorMsg({
				title: 'Export failed', 
				message: err.message ? err.message : 'Some error occured when trying to import your subscriptions to file.'
			});
		}
	}
	
	/**
	 * importFeeds(): 
	 * */
	function importFeeds(){
		
		try {
			opera.io.filesystem.browseForFile('', '', 
				function(mp) {
					if(mp === null) { return; }
					try {
						var fs =  mp.open(null, 'r');
		            	var text = fs.read( fs.bytesAvailable );
						var tmpFeeds = Parser.getChannels(text);
	
						Application.UI.overlay.showInfoDialog({
							title: 'Import successfull',
							message: "Your subscriptions has beed imported. In a moment list of news will be refreshed."
						});
						getFeeds(tmpFeeds);
					} catch( err ) {
			    		Application.UI.overlay.showErrorMsg({
							title: 'Import failed', 
							message: err.message ? err.message : 'Some error occured when trying to load subscriptions from selected file.'
						});
					}
	        	}, false, false );
    	} catch(err) {
    		Application.UI.overlay.showErrorMsg({
				title: 'Import failed', 
				message: err.message ? err.message : 'Some error occured when trying to load subscriptions from selected file.'
			});

    	}
	}
	
	/**
	 * load():
	 * */
	function load(){
		
		if( window.localSorage != 'undefined' ){
			var loadString = window.localStorage.getItem('feeds');
		}
		else {
			var loadString = widget.preferenceForKey('feeds')
		}
		if(!loadString){
			return;
		}
		
		var loadFeeds = eval( '(' + loadString + ')' ),
		    feed;
		for(var i = 0; i < loadFeeds.length; i++){
			try{
				feed = Feed.create(loadFeeds[i]);
				feeds.push(feed);
			} catch(err){
				//opera.postError( err + ': could not unserialize feed ' + loadFeeds[i] );
			}
		}
	}
	
	return {
		/**
		 * subscribes to events
		 * @method init
		 */
		init: function(){
			
			Application.UI.reader.init();
			Application.UI.overlay.init();
			load();
			
			if( !feeds.length ) {
				feeds = [ new Feed( 'http://my.opera.com/widgetsteam/xml/rss/blog/', [], { color: '#000000', title: 'Widgets team playground' }),
				          new Feed( 'http://www.opera.com/press/rss/', [], { color: '#CC0000', title: 'Opera Press Room' } )];
				getFeeds(feeds);
			} else {
				var items = [];
				for( var i=0;i < feeds.length; i++){
					items = items.concat(feeds[i].getItems());
				}
				Application.UI.reader.showItems(items);
			}			
			
			//var t = setInterval(getFeeds, 30000 );
		},
		
		loadFeeds: function(){
			Application.UI.reader.showFeeds(feeds);
		},
		
		showItems: function(){
			var items = [];
			for( var i=0;i < feeds.length; i++){
				items = items.concat(feeds[i].getItems());
			}
			Application.UI.reader.showItems(items);
		},
		
		loadItems: function(){
			//opera.postError( 'load feeds' );
			var x = this;
			if(activeRequests.length) x.abort();
			getFeeds();
		},
		
		/**
		 * aborts all active requests
		 * @method abort
		 */
		abort: function(){
			var xhr;
			while(activeRequests.length){
				xhr = activeRequests.pop();
				xhr.abort();
			}
		},

		/** 
		 * add feed to list or skip if feed with the same url exists
		 * @method add 
		 * @param {object} feed 
		 **/
		add: function(feed){
			
			var isModified = false,
				foundFeed,
				reader = this;
			
			foundFeed = findFeed(feed);
			
			if(foundFeed){
				//TODO display communique that feed exists
				Application.UI.reader.showFeedItems(foundFeed);
				return;
			}
			//getFeeds([feed]);
			feeds.push(feed);
			
	        Application.UI.overlay.showLoader( {title: 'Wait...', message: 'Retrieving RSS data...'} );

			getFeed(feed,function(feed,err){
				//opera.postError( 'feed retrieved ' + err );
				Application.UI.overlay.hideLoader();
				if(!err){
					Application.UI.reader.showFeedItems(feed);
					save();
				}
				else {
					//TODO display communique that url is wrong
					Application.UI.overlay.showErrorMsg({
						title: 'Error', 
						message: 'Could not retrieve data from feed. Check url address.'});
					var index = feeds.indexOf(feed);
					
					if(index != -1){
						feeds.splice(index,1);
					}
					reader.showItems();
				}
			});
			
		},
		
		/** 
		 * edit feed from list or skip if feed doesn't exist
		 * @method edit 
		 * @param {object} feed 
		 **/
		edit: function(feed,params){
			
			var isModified = false,
				oldUrl = feed.url,
				foundFeed,
				reader = this;
			
			foundFeed = findFeed(feed);
			
			if(foundFeed){
				
				if( params.color != feed.color ){
					feed.color = params.color;
					save();
					Application.UI.reader.showFeeds(feeds);
				}
				if( params.url != feed.url ) {
					feed.url = params.url;
					
					Application.UI.overlay.showLoader( {title: 'Wait...', message: 'Updateing RSS data...'} );
					getFeed(feed,function(feed,err){
						//opera.postError( 'feed retrieved ' + err );
						Application.UI.overlay.hideLoader();
						if(!err){
							Application.UI.reader.showFeedItems(feed);
							save();
							return true;
						}
						else {
							//TODO display communique that url is wrong
							Application.UI.overlay.showErrorMsg({
								title: 'Error', 
								message: 'Could not retrieve data from feed. Check url address.'});
							feed.url = oldUrl;
							return false;
						}
					});
				}
				return true;
			}
			//TODO: What if FEED does not exist! 
		},
		
		/** 
		 * @method remove 
		 * @param {Feed} feed this
		 **/
		remove: function(feed){
			
			var index = feeds.indexOf(feed);
			
			if(index != -1){
				
				feeds.splice(index,1);
				
				/*
				var items = [];
				for( var i=0;i < feeds.length; i++){
					items = items.concat(feeds[i].getItems());
				}
				Application.UI.reader.showItems(items);
				*/
				Application.UI.reader.showFeeds(feeds);
				save();
				
				//opera.postError( index + ' found ' + ' list of feeds length: ' + feeds.length );
			}
		},
		
		save: save,
		
		importFeeds: importFeeds,
		
		exportFeeds: exportFeeds
		
	};
})();
