
/**
 * creator for feed form UI, object responsible for adding new feed
 */
Application.UI.overlay = (function(){
	
	var feedForm = null;
	var loader   = null;
	
	/**
	 * 
	 * */
	function saveFeed(url,color){
		if (url.length) {
			if( !feedForm.feed ) {
				var feed = new Feed(url, [], {color: color});
				Application.Reader.add(feed);
			}
			else {
				return Application.Reader.edit(feedForm.feed, {url: url, color: color});
			}
			return true;
		}
		else {
			Application.UI.overlay.showErrorMsg({
				title: 'Error', 
				message: 'Write url address.'});
			return false;
		}
	}
	
	/**
	 * 
	 * */
	function hideOverlay(){
		var overlay = document.getElementById( 'overlay' );
		overlay.clear();
		overlay.addClass( 'hidden' );
	}
	
	/**
	 * 
	 * */
	function showOverlay(box){
		var overlay = document.getElementById( 'overlay' );
		overlay.clear(); // just in case if error,dialog remained
		if( box )overlay.appendChild( box );
		overlay.removeClass( 'hidden' );
	}
	
	/**
	 * 
	 * */
	function createInfoDialog(params) {
		
		var dialog 	 	 = document.createElement( 'div' ),
			message  	 = document.createElement( 'p' ),
			buttonOk 	 = document.createElement( 'span' ),
			title 	 	 = document.createElement( 'h3' );
	
		title.appendChild( document.createTextNode( params.title ) );
		message.appendChild( document.createTextNode( params.message ) );
		
		dialog.appendChild( title );
		dialog.appendChild( message );
		
		buttonOk.appendChild( document.createTextNode( 'Ok' ) );
		buttonOk.addEventListener( 'click', function(){
			hideOverlay();
		}, false );
		buttonOk.addClass( 'button' );
		
		dialog.appendChild( buttonOk );
		dialog.id = 'confirmDialog';
		dialog.addClass( 'overlayBox' );
		dialog.addClass( 'box' );
		return dialog;
	}
	
	/**
	 * 
	 * */
	function createConfirmDialog(params) {
		
		var dialog 	 	 = document.createElement( 'div' ),
			message  	 = document.createElement( 'p' ),
			buttonOk 	 = document.createElement( 'span' ),
			buttonCancel = document.createElement( 'span' ),
			title 	 	 = document.createElement( 'h3' );
	
		title.appendChild( document.createTextNode( params.title ) );
		message.appendChild( document.createTextNode( params.message ) );
		
		dialog.appendChild( title );
		dialog.appendChild( message );
		
		buttonCancel.appendChild( document.createTextNode( 'Cancel' ) );
		buttonCancel.addEventListener( 'click', params.onCancel, false );
		buttonCancel.addClass( 'button' );
		buttonOk.appendChild( document.createTextNode( 'Ok' ) );
		buttonOk.addEventListener( 'click', function(){
			params.onOk();
			hideOverlay();
		}, false );
		buttonOk.addClass( 'button' );
		
		dialog.appendChild( buttonOk );
		dialog.appendChild( buttonCancel );
		dialog.id = 'confirmDialog';
		dialog.addClass( 'overlayBox' );
		dialog.addClass( 'box' );
		return dialog;
	}

	/**
	 * 
	 * */
	function createLoader(params){
		
		var loader 	 	 = document.createElement( 'div' ),
			message  	 = document.createElement( 'p' ),
			title 	 	 = document.createElement( 'h3' );
		
		title.appendChild( document.createTextNode( params.title ? params.title : 'Loading' ) );
		message.appendChild( document.createTextNode( params.message ? params.message : 'content is loading...' ) );
	
		loader.appendChild( title );
		loader.appendChild( message );
	
		loader.id = 'loader';
		loader.addClass( 'overlayBox' );
		loader.addClass( 'box' );
		return loader;
		
	}
	
	/**
	 * 
	 * */
	function createErrorMsg(params){
		
		var error 	 	 = document.createElement( 'div' ),
			buttonClose  = document.createElement( 'span' ),
			message  	 = document.createElement( 'p' ),
			title 	 	 = document.createElement( 'h3' );

		title.appendChild( document.createTextNode( params.title ? params.title : 'Error' ) );
		message.appendChild( document.createTextNode( params.message ? params.message : 'Error appears...' ) );
	
		buttonClose.appendChild( document.createTextNode( 'Close' ) );
		buttonClose.addEventListener( 'click', hideOverlay, false );
		buttonClose.addClass( 'button' );
		
		error.appendChild( title );
		error.appendChild( message );
		error.appendChild( buttonClose );
		
		error.id = 'error';
		error.addClass( 'errorBox' );
		error.addClass( 'box' );
		
		return error;
		
	}
	
	/**
	 * 
	 * */
	function createColorSelect(){
		var col 	 	 = document.createElement( 'span' ),
			colors 		 = [ '#f5890b', 
			       		     '#f5470b', 
			       		     '#a62a45', 
			       		     '#c3bd21',
			       		     '#f2c90c',
			       		     '#31c1db',
			       		     '#1e60a9',
			       		     '#1da297',
			       		     '#59a812',
			       		     '#8f8b76' ],
			colorSelect  = document.getElementById('colorSelect'),
			colorInput   = document.getElementById('feedColor');
		
		for(var i=0; i<colors.length;i++){
			var item = col.cloneNode(false);
			item.textContent = colors[i];
			item.style = 'background-color: ' + colors[i];
			item.addEventListener( 'click', function(e){
				var el = e.target;
				colorInput.value = ( el.textContent );
				
				items = colorSelect.getElementsByTagName( 'span' );
				for( var i=0; i< items.length; i++ ){
					items[i].removeClass( 'selected' );
				}
				
				el.addClass( 'selected' );
			}, false );
			colorSelect.appendChild(item);
		}
	}
	return {
		
		init: function(){
			var self = this;
			feedForm = document.getElementById( 'feedForm' );
            var url = document.getElementById('feedUrl');
            var feedColor = document.getElementById('feedColor');
            var subm = document.getElementById('saveFeedBtn');
            var cancel = document.getElementById('cancelFormBtn');
            createColorSelect();
            
            cancel.addEventListener( 'click', 
					function(){ 
            			self.hideForm(); 
        			}, 
				false );
            
			//click listener for button
			subm.addEventListener('click', function(){ 
                if(saveFeed(url.value,feedColor.value)) self.hideForm(); 
            }, false );
			
			feedColor.addEventListener('change',function(){
				ocument.getElementById('feedColor').color.hidePicker()
			}, false);
			//enter listener for input
			url.addEventListener('keydown', function(e){ 
                if (e.keyCode == 13) {
                	if(saveFeed(url.value,feedColor.value)) self.hideForm();
				}
            }, false );
            
		},
		
		showForm: function(feed){
			
			var formTitle = document.getElementById('formTitle');
			feedForm.removeClass( 'hidden' );
			feedForm.feed = feed;
			document.getElementById('feedUrl').value = feed ? feed.url : '';
			document.getElementById('feedColor').value = feed ? feed.color : '';
			colorSelect  = document.getElementById('colorSelect');
			colors = colorSelect.getElementsByTagName('span');
			for( var i=0; i< colors.length; i++ ){
				if( feed && colors[i].textContent != feed.color )
					colors[i].removeClass('selected');
			}
			formTitle.textContent = ( feed ? 'edit' : 'add' ) + ' rss channel';
		},
				
		hideForm: function(){
			feedForm.addClass( 'hidden' );
		},
		showConfirmDialog: function( params){
			var dialog = createConfirmDialog( params );
			showOverlay(dialog);
		},
		showInfoDialog: function( params){
			var dialog = createInfoDialog( params );
			showOverlay(dialog);
		},
		showLoader: function(params){
			var loader = createLoader(params);
			showOverlay(loader);
		},
		showErrorMsg: function(params){
			var error = createErrorMsg(params);
			showOverlay(error);
		},
		removeConfirmDialog: hideOverlay,
		hideLoader: hideOverlay
		
		
	}
})();
