
/**
 * creator for explorer UI, object responsible for songs browser view
 */
Application.UI.reader = (function(){
    
	/**
	 * time formatting 
	 * @method formatTime
	 * @param {timestamp} d
	 * @returns {String} with formatted date
	 * */
	function formatTime(d){
		
		var date = new Date(),
			diff = ((date.getTime() - d) / 1000),
			day_diff = Math.floor(diff / 86400);
			
		//opera.postError( 'day diff: ' + day_diff + ' - ' + d.toString()+'');
		if ( isNaN(day_diff) || day_diff < 0  )
			return '';
			
		return day_diff == 0 && 
			( 
					diff < 60 && "just now" ||
					diff < 120 && "1 minute ago" ||
					diff < 3600 && Math.floor( diff / 60 ) + " minutes ago" ||
					diff < 7200 && "1 hour ago" ||
					diff < 86400 && Math.floor( diff / 3600 ) + " hours ago"
			)||
			
			day_diff == 1 && "Yesterday" ||
			day_diff < 7 && day_diff + " days ago" ||
			day_diff < 31 && Math.ceil( day_diff / 7 ) + " weeks ago" || 
			day_diff >= 31 && "few months ago";
	}
	
	/**
	 * updating formatted pubDate of every displayed item
	 * */
	function updateTime(){
		var list = document.getElementById( 'item_list' );
		if( list ){
			var items = list.getElementsByTagName( 'h6' );
			for( var i=0; i<items.length; i++){
				var item = items[i];
				if( item.hasClass( 'pubDate' ) ){
					item.textContent = formatTime( item.pubDate );
				}
			}
		}
	}
	
	/**
	 * items sorting callback
	 * sorts by pubDate
	 * @method dateCompare
	 * @private
	 * @param {Object} a
	 * @param {Object} b
	 */
	function dateCompare(a,b){
		//if both are of the same type, compare names
		if(a.pubDate > b.pubDate){
			return -1;
		}
		if(a.pubDate < b.pubDate){
			return 1;
		}
		
		return 0;
	}
	
	/**
	 * show items of passed Feed
	 * @param {Object} feed
	 */
	function showFeed(feed){
        
		var title = document.getElementById( 'titleTxt' ),
			icon = document.getElementById( 'titleIcon' ),
			itemsList = document.getElementById( 'item_list' );
			items = feed.getItems();
		
		itemsList.addClass( 'feedList' );
		icon.addClass( 'clickable' );
        displayItems(items);
        title.textContent = feed.title;
        if(feed.color){
        	icon.style = 'background-color: ' + feed.color;
        }
    }
	
	/**
	 * show passed items from all feeds
	 * @param {Array} of items
	 */
	function showItems(items){
		var title = document.getElementById( 'titleTxt' ),
			icon = document.getElementById( 'titleIcon' ),
			itemsList = document.getElementById( 'item_list' );
		
		itemsList.removeClass( 'feedList' );
	    displayItems(items);
	    title.textContent = 'all feeds';
    	icon.style = '';
    	icon.removeClass( 'clickable' );
	}
	
	/**
	 * show passed feeds
	 * @param {Array} of feeds
	 */
	function showFeeds(feeds){
		
		var itemList = document.getElementById('item_list'),
			feedList = document.getElementById('feed_list'),
			icon = document.getElementById( 'titleIcon' ),
			title = document.getElementById( 'titleTxt' ),
			fake  = document.createElement('li'),
			elem;
		fake.addClass( 'fake' );
		feedList.innerHTML = '';
		icon.removeClass( 'clickable' )
		
		if (feeds.length) {
			for (var i = 0; i < feeds.length; i++) {
				elem = createFeed(feeds[i]);
				addFeedListener( elem, feeds[i] );
				feedList.appendChild(elem);
			}
			feedList.appendChild( fake );
		}
		itemList.addClass('hidden');
		feedList.removeClass('hidden');
		title.textContent = 'feeds';
		
		if( Application.isFileSupported() ){
			document.getElementById( 'importExportMenu' ).removeClass( 'hidden' );
		}
	}
	
	/**
	 * creates html object with item content
	 * @param {Item} item
	 * @return {Object} html item
	 * */
	function createItem(item){
		
		var elem 	= document.createElement( 'li' ),
			icon    = document.createElement( 'span' ),
			feed    = document.createElement( 'em' ),
			update  = document.createElement( 'h6' ),
			link 	= document.createElement( 'a' ),
			header 	= document.createElement( 'div' ),
			title   = document.createElement( 'h5' ),
			content = document.createElement( 'div' ),
			date    = document.createElement( 'h6' ),
			description = document.createElement( 'div' ) ;
			
		update.appendChild( document.createTextNode( formatTime(item.pubDate) + ' ' ));
		update.pubDate = item.pubDate;
		update.addClass('pubDate');
		
		header.appendChild( update );
		title.appendChild( document.createTextNode( item.title ) );
		header.appendChild( title );
		
		content.addClass( 'content' );
		description.innerHTML = item.content;//appendChild( document.createTextNode( item.content ) );
		description.addClass( 'description' );
		content.appendChild( description );
		link.addClass( 'button' );
		link.setAttribute('href', item.url);
		link.appendChild( document.createTextNode( 'view source' ));
		link.addEventListener( 'click', function(e){
			e.stopPropagation();
		}, false );
		content.appendChild( link );
		
		icon.addClass( 'icon' );
		if( item.feed ){
			if( item.feed.title ) {
				feed.appendChild( document.createTextNode( item.feed.title ) );
				icon.appendChild( feed );
			}
			if( item.feed.color ){
				icon.style="background-color: " + item.feed.color;
			}
			addFeedListener( icon, item.feed );
		}
		elem.addClass('item');
		if( item.read ) {
			elem.addClass('read');
		}
		elem.appendChild( icon );
		elem.appendChild( header );
		elem.appendChild( content );
		
		return elem;
	}
	
	/**
	 * creates html object with feed content
	 * @param {Object} feed
	 * @return {Object} html prepared for feed element
	 * */
	function createFeed(feed){
		
		var elem 	= document.createElement( 'li' ),
			update  = document.createElement( 'small' ),
			link 	= document.createElement( 'a' ),
			header 	= document.createElement( 'h3' ),
			content = document.createElement( 'div' ),
			date    = document.createElement( 'h6' ),
			description = document.createElement( 'p' ),
			remBtn  = document.createElement( 'span' ),
			editBtn = document.createElement( 'span' ),
			unread  = document.createElement( 'span' );
		
		editBtn.className = 'edit';
		editBtn.appendChild( document.createTextNode( 'edit' ) );
		addEditFeedListener( editBtn, feed );
		header.appendChild( editBtn );

		remBtn.className = 'remove';
		remBtn.appendChild( document.createTextNode( 'delete' ) );
		addDelFeedListener( remBtn, feed );
		header.appendChild( remBtn );

		header.appendChild( document.createTextNode( feed.title ) );
		unread.appendChild( document.createTextNode(' (' + feed.getUnreadNum() + ') ' ));
		header.appendChild( unread );
		if( feed.color ){
			header.style="background-color: " + feed.color;
		} else {
			elem.addClass('colorless');
		}
		description.appendChild( document.createTextNode( feed.description ) );
		content.appendChild( description );
		content.addClass( 'content' );
		date.appendChild( document.createTextNode( 'Last updated: ' + feed.lastUpdate ) );
		content.appendChild( date );
		
		link.setAttribute('href', feed.url);
		link.addClass( 'button' );
		link.appendChild( document.createTextNode( 'view source' ));
		link.addEventListener( 'click', function(e){
			e.stopPropagation();
		}, false );
		content.appendChild( link );
		
		elem.addClass('feed');
		if( !feed.unreadNum ) {
			elem.addClass('newItems');
		}
		
		elem.appendChild( header );
		elem.appendChild( content );

		return elem;
	}
	
	/**
	 * displays passed items
	 * @param {Array} of items
	 */
	function displayItems(items){
		
		items.sort(dateCompare);
		
		var itemList = document.getElementById('item_list'),
			feedList = document.getElementById('feed_list'),
			fake  = document.createElement('li');
	
		fake.addClass( 'fake' );
		itemList.innerHTML = '';
		
		var elem;
		
		if (items.length) {
			for (var i = 0; i < items.length; i++) {
				elem = createItem(items[i]);
				addItemListener(elem, items[i]);
				itemList.appendChild( elem );
			}
			itemList.appendChild( fake );
		}
		feedList.addClass( 'hidden' );
		itemList.removeClass( 'hidden' );
		document.getElementById( 'importExportMenu' ).addClass( 'hidden' );
	}
	
	/**
	 * 
	 * @method addItemListener
	 * @private
	 * @param {Object} elem
	 * @param {Object} item
	 */
	function addItemListener(elem, item){
		
		elem.addEventListener('click',function(e){
			//opera.postError(e.target.nodeName);
			if( e.target.nodeName == 'button' ) return;
			
			var isRead = item.read;
			if( !isRead ) {
				item.markAsRead();
				elem.addClass( 'read' );
				Application.Reader.save();
			}
			var isOpened = elem.hasClass('active');
			
			var list = document.getElementById( 'item_list' ),
				items = list.getElementsByTagName( 'li' );
			for( var i=0; i< items.length; i++ ){
				items[i].removeClass( 'active' );
			}
			
			if( isOpened ){
				elem.removeClass( 'active' );
			} else {
				elem.addClass( 'active' );
			}
		}, false);
		
	}
	
	/**
	 * 
	 * @method addFeedListener
	 * @private
	 * @param {Object} elem
	 * @param {Object} feed
	 */
	function addFeedListener(elem, feed){
		
		elem.addEventListener('click',function(e){
			e.stopPropagation();
			feed.get(function(){
				showFeed(this);
			});
		},false);
	}
	
	/**
	 * 
	 * @method addRemFeedListener
	 * @private
	 * @param {Object} elem
	 * @param {Object} feed
	 */
	function addDelFeedListener(elem, feed){
		
		elem.addEventListener('click',function(e){
			e.stopPropagation();
			var params = {
				title: 'Removing feed',
				message: "Are you sure that you wan't remove channel " + '"' + feed.title + '"?',
				onOk: function(){
					Application.Reader.remove( feed );
				},
				onCancel: function(){
					Application.UI.overlay.removeConfirmDialog();
				}
			};
			Application.UI.overlay.showConfirmDialog(params);//Reader.remove(feed);
		},false);
	}
	
	/**
	 * 
	 * @method addEditFeedListener
	 * @private
	 * @param {Object} elem
	 * @param {Object} feed
	 */
	function addEditFeedListener(elem, feed){
		
		elem.addEventListener('click',function(e){
			e.stopPropagation();
			Application.UI.overlay.showForm(feed);
			
		},false);
	}
	
    return {
        
		init: function(){
			document.getElementById( 'updateBtn' ).addEventListener( 'click', 
					function(){ Application.Reader.loadItems(); }, 
				false );
			
			document.getElementById( 'importBtn' ).addEventListener( 'click', 
					function(){ Application.Reader.importFeeds(); }, 
				false );
			
			document.getElementById( 'exportBtn' ).addEventListener( 'click', 
					function(){ Application.Reader.exportFeeds(); }, 
				false );
			
			document.getElementById( 'feedsBtn' ).addEventListener( 'click', 
				function(){ 
					if( document.getElementById( 'item_list' ).hasClass( 'hidden' ) ){
							Application.Reader.showItems(); 
				
					} else {
							Application.Reader.loadFeeds(); 
					}
				}, 
			false );
			
			document.getElementById( 'titleIcon' ).addEventListener( 'click', 
				function(){ 
					if( document.getElementById( 'item_list' ).hasClass( 'feedList' ) ){
						Application.Reader.showItems(); 
					}
				}, 
			false );
			
			document.getElementById( 'addBtn' ).addEventListener( 'click', 
					function(){ Application.UI.overlay.showForm(); }, 
				false );
			
			t = setInterval( updateTime, 60*1000 );
		},
		
        
        /**
         * displays items 
         * @method showItems
         * @param {Array} of items to display
         */
        showItems: showItems,
        
        /**
         * displays list of feeds
         * @method showItems
         * @param {Array} of Feeds to display
         */
        showFeeds: showFeeds,
        
        /**
         * displays items from passed feed
         * @method showFeedItems
         * @param {Object} Feed
         */
        showFeedItems: showFeed
    
    };
})();
